package provide DS9 1.0

# Public Functions

proc CreateColorbar {} {
    global colorbar
    global ds9

    set colorbar(name) \
	[canvas $ds9(main).cmap -width $colorbar(width) \
	     -height $colorbar(height) -bd 2 -relief groove -insertofftime 0]
    switch -- $ds9(visual) {
	pseudocolor {
	    $colorbar(name) create colorbar$ds9(visual)$ds9(depth) \
		-private $colorbar(private) \
		-width $colorbar(width) -height $colorbar(height) -anchor nw
	}
	truecolor {
	    $colorbar(name) create colorbar$ds9(visual)$ds9(depth) \
		-width $colorbar(width) -height $colorbar(height) \
		-anchor nw -colors 2048
	}
    }

    # And set the two windows that where created before the 
    # colorbar was created, to the new colormap

    colorbar set colormap window $ds9(main)

    # This is a kluge, but it works. We want to reassign the bg color 
    # for $ds9(main) from the old colormap to the new colormap

    $ds9(main) configure -background [$ds9(main) cget -background]
}

proc InitColorbar {} {
    global colorbar

    colorbar map $colorbar(map)
    colorbar invert $colorbar(invert)
    bind $colorbar(name) <Configure> [list UpdateColorbarGeometry]
}

proc LoadColormap {} {
    global ds9
    global colorbar

    set fileName [OpenFileDialog $ds9(top) ds9(lastColormapName)]
    if {$fileName != ""} {
	colorbar load $fileName
	set id [colorbar get id]
	set colorbar(map) [colorbar get name]
	.menuBar.color insert $colorbar(count) radiobutton \
	    -label "$colorbar(map)" \
	    -variable colorbar(map) -command "ChangeColormapID $id"
	incr colorbar(count)
    }
}

proc SaveColormap {} {
    global ds9

    set init [colorbar get file name]
    set fileName [SaveFileDialog $ds9(top) init]
    if {$fileName != ""} {
	colorbar save $fileName
    }
}

proc ButtonColormap {x y} {
    global current

    if {$current(frame) != ""} {
	$current(frame) colormap begin
    }
}

proc MotionColormap {x y} {
    global canvas
    global colorbar
    global current

    # X sets bias
    set cursorX \
	[expr double($x-$canvas(width)/2+$colorbar(speed)/2)/$colorbar(speed)]
    # Y set contrast
    set cursorY [expr double($y)/$colorbar(speed) * $colorbar(mult)]

    colorbar adjust $cursorY $cursorX
    if {$current(frame) != ""} {
	$current(frame) colormap motion [colorbar get colormap]
    }

    UpdateColormapDialog
}

proc ReleaseColormap {x y} {
    global current

    if {$current(frame) != ""} {
	$current(frame) colormap end [colorbar get colormap]
    }

    UpdateColormapDialog
}

proc CreateColorMenus {} {
    global colorbar

    set id [colorbar list id]

    foreach i $id {
	set name [colorbar get name $i]
	.menuBar.color insert $colorbar(count) radiobutton \
	    -label "$name" -variable colorbar(map) \
	    -command "ChangeColormapID $i"
	.menuBar.prefs.color insert $colorbar(count) radiobutton \
	    -label "$name" -variable prefs(colorbarmap)
	incr colorbar(count)
    }
}

proc UpdateColorbarGeometry {} {
    global colorbar

    set colorbar(width) [expr [winfo width $colorbar(name)]-2]
    set colorbar(height) [expr [winfo height $colorbar(name)]-2]

    colorbar configure -width $colorbar(width) -height $colorbar(height)
}

proc ChangeColormapID {id} {
    global current
    global colorbar
    
    SetWatchCursor
    colorbar map $id
    if {$current(frame) != "" } {
	$current(frame) colormap [colorbar get colormap]
	set colorbar(map) [colorbar get name]
	set colorbar(invert) [colorbar get invert]
    }
    UnsetWatchCursor

    UpdateColormapDialog
}

proc ChangeColormap {} {
    global current
    global colorbar
    
    SetWatchCursor
    colorbar map $colorbar(map)
    if {$current(frame) != "" } {
	$current(frame) colormap [colorbar get colormap]
	set colorbar(invert) [colorbar get invert]
    }
    UnsetWatchCursor

    UpdateColormapDialog
}

proc MatchColorbars {} {
    global ds9
    global current
    global colorbar

    SetWatchCursor
    foreach f $ds9(frames) {
	if {$f != $current(frame)} {
	    $f colormap [colorbar get colormap]
	}
    }
    UnsetWatchCursor
}

proc InvertColorbar {} {
    global colorbar
    global current

    SetWatchCursor
    colorbar invert $colorbar(invert)
    if {$current(frame) != "" } {
	$current(frame) colormap [colorbar get colormap]
    }
    UnsetWatchCursor

    UpdateColormapDialog
}

proc ColormapDialog {} {
    global colorbar
    global ds9

    # see if we already have a window visible

    if [winfo exist $colorbar(top)] {
	return
    }

    set w $colorbar(top)
    set title "Contrast/Bias"
    set length 300

    # create the window

    toplevel $w -colormap $ds9(main)
    wm title $w $title
    wm iconname $w $title
    wm group $w $ds9(top)
    wm protocol $w WM_DELETE_WINDOW ColormapDestroyDialog

    UpdateColormapDialog

    frame $w.cb  -relief groove -borderwidth 2
    frame $w.buttons -relief groove -borderwidth 2
    pack $w.cb $w.buttons -ipadx 4 -ipady 4 -fill x -expand true

    frame $w.cb.contrast
    scale $w.cb.contrast.slider -from 0 -to 10 -length $length \
	-variable colorbar(contrast) -orient horizontal -label "Contrast" \
	-tickinterval 2 -showvalue false -resolution .1 \
	-command AdjustColormap
    entry $w.cb.contrast.value -textvariable colorbar(contrast) -width 7 \
	-relief sunken -bd 2
    pack $w.cb.contrast.slider $w.cb.contrast.value -side left -padx 4

    frame $w.cb.bias
    scale $w.cb.bias.slider -from 0 -to 1 -length $length \
	-variable colorbar(bias) -orient horizontal -label "Bias" \
	-tickinterval .2 -showvalue false -resolution .01 \
	-command AdjustColormap
    entry $w.cb.bias.value -textvariable colorbar(bias) -width 7 \
	-relief sunken -bd 2
    pack $w.cb.bias.slider $w.cb.bias.value -side left -padx 4

    pack $w.cb.contrast $w.cb.bias -side top

    button $w.buttons.apply -text "Apply" -command {AdjustColormap 1}
    button $w.buttons.close -text "Close" -command ColormapDestroyDialog
    pack $w.buttons.apply $w.buttons.close -side left -padx 10 -expand true
}

proc ColormapDestroyDialog {} {
    global colorbar

    destroy $colorbar(top)
}

proc AdjustColormap {value} {
    # value is ignored
    global colorbar
    global current

    colorbar adjust $colorbar(contrast) $colorbar(bias)
    if {$current(frame) != ""} {
	$current(frame) colormap motion [colorbar get colormap]
    }
}

proc UpdateColormapDialog {} {
    global colorbar

    if {[winfo exist $colorbar(top)]} {
	set colorbar(contrast) [colorbar get contrast]
	set colorbar(bias) [colorbar get bias]
    }
}
