package provide DS9 1.0

proc CenterCurrentFrame {} {
    global current

    CenterFrame $current(frame)
}

proc CenterAllFrame {} {
    global ds9

    foreach f $ds9(frames) {
	CenterFrame $f
    }
}

proc CenterFrame {which} {
    if {$which != ""} {
	SetWatchCursor
	$which center
	UpdateGraphXAxis
	UnsetWatchCursor

	UpdatePanZoomMenu
    }
}

proc Pan {x y sys} {
    global current

    if {$current(frame) != ""} {
	if {$sys == ""} {
	    set sys image
	}
	SetWatchCursor
	$current(frame) pan $x $y $sys
	UpdateGraphXAxis
	UnsetWatchCursor
    }
}

proc PanTo {x y sys} {
    global current

    if {$current(frame) != ""} {
	if {$sys == ""} {
	    set sys image
	}
	SetWatchCursor
	$current(frame) pan to $x $y $sys
	UpdateGraphXAxis
	UnsetWatchCursor
    }
}

proc ButtonPan {which x y} {
    global pan

    switch $pan(mode) {
	click {}
	drag {
	    $which pan motion begin $x $y
	}
    }
}

proc MotionPan {which x y} {
    global pan

    switch $pan(mode) {
	click {}
	drag {
	    $which pan motion $x $y
	}
    }
}

proc ReleasePan {which x y} {
    global pan

    switch $pan(mode) {
	click {
	    SetWatchCursor
	    $which pan to $x $y
	}
	drag {
	    $which pan motion end $x $y

	    # we need to delay setting the cursor here, since set cursor causes
	    # an update, so the display will flash

	    SetWatchCursor
	}
    }
    UpdateGraphXAxis
    UnsetWatchCursor
}

proc ZoomToFit {} {
    global current

    if {$current(frame) != ""} {
	SetWatchCursor
	$current(frame) zoom to fit
	set current(zoom) [$current(frame) get zoom]
	UpdateGraphXAxis
	UnsetWatchCursor
    }
}

proc ChangeZoom {} {
    global current

    if {$current(frame) != ""} {
	SetWatchCursor
	$current(frame) zoom to $current(zoom)
	UpdateGraphXAxis
	UnsetWatchCursor
    }
}

proc Zoom {value} {
    global current

    if {$current(frame) != ""} {
	SetWatchCursor
	$current(frame) zoom $value
	set current(zoom) [$current(frame) get zoom]
	UpdateGraphXAxis
	UnsetWatchCursor
    }
}

proc ButtonZoom {which x y} {
    global current

    if {$current(frame) != ""} {
	SetWatchCursor
	$which zoom 2 about $x $y
	if {$which == $current(frame)} {
	    set current(zoom) [$current(frame) get zoom]
	}
	UpdateGraphXAxis
	UnsetWatchCursor
    }
}

proc ShiftZoom {which} {
    global current

    if {$current(frame) != ""} {
	SetWatchCursor
	$which zoom .5
	if {$which == $current(frame)} {
	    set current(zoom) [$current(frame) get zoom]
	}
	UpdateGraphXAxis
	UnsetWatchCursor
    }
}

proc ChangeOrient {} {
    global current

    if {$current(frame) != ""} {
	SetWatchCursor
	$current(frame) orient $current(orient)
	UpdateGraphXAxis
	UnsetWatchCursor
    }
}

proc Rotate {value} {
    global current

    if {$current(frame) != ""} {
	SetWatchCursor
	$current(frame) rotate $value
	set current(rotate) [$current(frame) get rotate]
	UpdateGraphXAxis
	UnsetWatchCursor
    }
}

proc ChangeRotate {} {
    global current

    if {$current(frame) != ""} {
	SetWatchCursor
	$current(frame) rotate to $current(rotate)
	UpdateGraphXAxis
	UnsetWatchCursor
    }
}

proc ButtonRotate {which x y} {
    global rotate

    $which rotate motion begin
    set rotate(x) $x
}

proc MotionRotate {which x y} {
    global rotate
    global cursor
    global angleNameValue

    $which rotate motion [expr double($rotate(x)-$x)/$cursor(speed) * 180.]
    set angleNameValue [$which get rotate fixed]
}

proc ReleaseRotate {which x y} {
    global angleNameValue
    global current

    $which rotate motion end
    set angleNameValue [$which get rotate fixed]
    set current(rotate) [$which get rotate fixed]

    # we need to delay setting the cursor here, since set cursor causes
    # an update, so the display will flash

    SetWatchCursor
    UpdateGraphXAxis
    UnsetWatchCursor
}

proc PanZoomDialog {} {
    global panZoom
    global menu
    global ds9

    # see if we already have a window visible

    if [winfo exist $panZoom(top)] {
	return
    }

    set w $panZoom(top)
    set title "Pan Zoom Rotate"

    # create the window

    toplevel $w -colormap $ds9(main)
    wm title $w $title
    wm iconname $w $title
    wm group $w $ds9(top)
    wm protocol $w WM_DELETE_WINDOW PanZoomDestroyDialog

    $w configure -menu $panZoom(mb)

    menu $panZoom(mb) -tearoff 0
    $panZoom(mb) add cascade -label File -menu $panZoom(mb).file
    $panZoom(mb) add cascade -label Edit -menu $panZoom(mb).edit
    $panZoom(mb) add cascade -label Coordinate -menu $panZoom(mb).coord

    menu $panZoom(mb).file -tearoff 0 -selectcolor $menu(selectcolor)
    $panZoom(mb).file add command -label "Apply" -command PanZoomApplyDialog
    $panZoom(mb).file add separator
    $panZoom(mb).file add command -label "Close" -command PanZoomDestroyDialog

    menu $panZoom(mb).edit -tearoff 0 -selectcolor $menu(selectcolor)
    $panZoom(mb).edit add command -label "Copy" -state disabled
    $panZoom(mb).edit add command -label "Paste" -state disabled
    $panZoom(mb).edit add command -label "Clear" -state disabled

    menu $panZoom(mb).coord -tearoff 0 -selectcolor $menu(selectcolor)
    $panZoom(mb).coord add radiobutton -label "Equatorial B1950" \
	-variable panZoom(system) -value fk4 \
	-command PanZoomDialogCoord
    $panZoom(mb).coord add radiobutton -label "Equatorial J2000" \
	-variable panZoom(system) -value fk5 \
	-command PanZoomDialogCoord
    $panZoom(mb).coord add radiobutton -label "ICRS" \
	-variable panZoom(system) -value icrs \
	-command PanZoomDialogCoord
    $panZoom(mb).coord add radiobutton -label "Galactic" \
	-variable panZoom(system) -value galactic \
	-command PanZoomDialogCoord
    $panZoom(mb).coord add radiobutton -label "Ecliptic" \
	-variable panZoom(system) -value ecliptic \
	-command PanZoomDialogCoord
    $panZoom(mb).coord add separator
    $panZoom(mb).coord add radiobutton -label "Linear" \
	-variable panZoom(system) -value linear \
	-command PanZoomDialogCoord
    $panZoom(mb).coord add separator
    $panZoom(mb).coord add radiobutton -label "Detector" \
	-variable panZoom(system) -value detector \
	-command PanZoomDialogCoord
    $panZoom(mb).coord add radiobutton -label "Amplifier" \
	-variable panZoom(system) -value Amplifier \
	-command PanZoomDialogCoord
    $panZoom(mb).coord add radiobutton -label "Physical" \
	-variable panZoom(system) -value physical \
	-command PanZoomDialogCoord
    $panZoom(mb).coord add radiobutton -label "Image" \
	-variable panZoom(system) -value image \
	-command PanZoomDialogCoord
    $panZoom(mb).coord add separator
    $panZoom(mb).coord add radiobutton -label "Degrees" \
	-variable panZoom(format) -value degrees \
	-command PanZoomDialogCoord
    $panZoom(mb).coord add radiobutton -label "Sexagesimal" \
	-variable panZoom(format) -value sexagesimal \
	-command PanZoomDialogCoord

    frame $w.param -relief groove -borderwidth 2
    frame $w.buttons -relief groove -borderwidth 2
    pack $w.param $w.buttons -fill x -ipadx 4 -ipady 4

    label $w.param.zoomtitle -text "Zoom"
    entry $w.param.zoomvalue -textvariable panZoom(zoom) -width 14 \
	-relief sunken -bd 2

    label $w.param.rottitle -text "Rotate"
    entry $w.param.rotvalue -textvariable panZoom(rotate) -width 14 \
	-relief sunken -bd 2
    label $w.param.rottitle2 -text "degrees"

    label $w.param.pantitle -text "Pan"
    set panZoom(xname) [label $w.param.panxtitle -text "" -width 1]
    entry $w.param.panx -textvariable panZoom(x) -width 14 \
	-relief sunken -bd 2
    set panZoom(yname) [label $w.param.panytitle -text "" -width 1]
    entry $w.param.pany -textvariable panZoom(y) -width 14 \
	-relief sunken -bd 2
    label $w.param.pansystem -textvariable panZoom(system) \
	-width 10 -relief groove -bd 2
    
    grid rowconfigure $w.param 0 -pad 4
    grid rowconfigure $w.param 1 -pad 4
    grid rowconfigure $w.param 2 -pad 4

    grid $w.param.zoomtitle -row 0 -column 0 -columnspan 2 -padx 4 -sticky w
    grid $w.param.zoomvalue -row 0 -column 2 -padx 4 -sticky w

    grid $w.param.rottitle -row 1 -column 0 -columnspan 2 -padx 4 -sticky w
    grid $w.param.rotvalue -row 1 -column 2 -padx 4 -sticky w
    grid $w.param.rottitle2 -row 1 -column 3 -columnspan 2 -padx 4 -sticky w

    grid $w.param.pantitle -row 2 -column 0 -padx 4 -sticky w
    grid $panZoom(xname) -row 2 -column 1 -padx 4 -sticky w
    grid $w.param.panx -row 2 -column 2 -padx 4 -sticky w
    grid $panZoom(yname) -row 2 -column 3 -padx 4 -sticky w
    grid $w.param.pany -row 2 -column 4 -padx 4 -sticky w
    grid $w.param.pansystem -row 2 -column 5 -padx 4 -sticky w

    button $w.buttons.apply -text "Apply" -command PanZoomApplyDialog
    button $w.buttons.close -text "Close" -command PanZoomDestroyDialog
    pack $w.buttons.apply $w.buttons.close -side left -padx 10 -expand true

    $w.param.zoomvalue select range 0 end

    UpdatePanZoomDialog
}

proc PanZoomApplyDialog {} {
    global current
    global panZoom

    if {$current(frame) != ""} {
	set current(zoom) $panZoom(zoom)
	set current(rotate) $panZoom(rotate)

	SetWatchCursor
	$current(frame) zoom to $current(zoom) about \
	    $panZoom(x) $panZoom(y) $panZoom(system)
	$current(frame) rotate to $current(rotate)
	UpdateGraphXAxis
	UnsetWatchCursor
    }
}

proc PanZoomDestroyDialog {} {
    global panZoom

    destroy $panZoom(top)
    destroy $panZoom(mb)
}

proc PanZoomDialogCoord {} {
    global panZoom
    global current

    if {$current(frame) != ""} {
	set coord \
	    [$current(frame) get cursor $panZoom(system) $panZoom(format)]
	set panZoom(x) [lindex $coord 0]
	set panZoom(y) [lindex $coord 1]
    } else {
	set panZoom(x) {}
	set panZoom(y) {}
    }

    switch $panZoom(system) {
	fk4 -
	fk5 -
	icrs {
	    $panZoom(xname) configure -text "a" -font symbol
	    $panZoom(yname) configure -text "d" -font symbol
	}
	galactic {
	    $panZoom(xname) configure -text "l" \
		-font {-family times -slant italic}
	    $panZoom(yname) configure -text "b" \
		-font {-family times -slant italic}
	}
	ecliptic {
	    $panZoom(xname) configure -text "l" -font symbol
	    $panZoom(yname) configure -text "b" -font symbol
	}
	default {
	    $panZoom(xname) configure -text "X" -font default
	    $panZoom(yname) configure -text "Y" -font default
	}
    }
}

proc UpdatePanZoomDialog {} {
    global panZoom
    global current

    if {[winfo exist $panZoom(top)]} {
	if {$current(frame) != ""} {
	    set panZoom(zoom) [$current(frame) get zoom fixed]
	    set panZoom(rotate) [$current(frame) get rotate fixed]
	} else {
	    set panZoom(zoom) {}
	    set panZoom(rotate) {}
	}

	UpdateCoordMenu $panZoom(mb).coord 0 0 1
	AdjustCoordSystem panZoom(system)
	PanZoomDialogCoord
    }
}

proc UpdatePanZoomMenu {} {
    global current
    global wcs
    global buttons
    
    if {$current(frame) != ""} {
	.menuBar entryconfig Zoom -state normal
	$buttons(name).major.zoom configure -state normal

	if {[$current(frame) get fits count] > 0} {
	    .menuBar.zoom entryconfig "Zoom to Fit Frame" -state normal
	} else {
	    .menuBar.zoom entryconfig "Zoom to Fit Frame" -state disabled
	}

	set current(zoom) [$current(frame) get zoom]
	set current(rotate) [$current(frame) get rotate]
	set current(orient) [$current(frame) get orient]
	set wcs(align) [$current(frame) get wcs align]
    } else {
	.menuBar entryconfig Zoom -state disabled
	$buttons(name).major.zoom configure -state disabled

	set current(zoom) 1
	set current(rotate) 0
	set current(orient) none
	set wcs(align) 0
    }
}
