package provide DS9 1.0

proc RulerDialog {frame id} {
    global menu
    global marker
    global ds9

    # see if we already have a header window visible

    set w ".marker$id"
    set mb ".mb$id"

    if [winfo exist $w] {
	return
    }

    # init marker variables

    set marker($frame,$id,coord) image
    set marker($frame,$id,format) degrees
    set marker($frame,$id,distformat) pixels

    CoordRulerCB $frame $id
    TextMarkerCB $frame $id
    ColorMarkerCB $frame $id
    PropertyMarkerCB $frame $id
    FontMarkerCB $frame $id

    # ruler specific callbacks

    $frame marker $id callback move EditRulerCB $frame
    $frame marker $id callback edit EditRulerCB $frame
    $frame marker $id callback text TextMarkerCB $frame
    $frame marker $id callback color ColorMarkerCB $frame
    $frame marker $id callback property PropertyMarkerCB $frame
    $frame marker $id callback font FontMarkerCB $frame
    $frame marker $id callback delete DeleteRulerDialog $frame

    # define menus

    MBMarkerDialog $frame $mb
    ColorMenuMarkerDialog $frame $mb $id
    PropMenuRulerDialog $frame $mb $id
    CoordMenuMarkerDialog $frame $mb $id CoordRulerCB
    FontMenuMarkerDialog $frame $mb $id
    DistMenuMarkerDialog $frame $mb $id CoordRulerCB distformat
    $mb add cascade -label Distance -menu $mb.distformat

    # create window

    set type [string toupper "[$frame get marker $id type]"]
    toplevel $w -colormap $ds9(main)
    wm title $w $type
    wm iconname $w $type
    wm group $w $ds9(top)
    wm protocol $w WM_DELETE_WINDOW "CloseRulerDialog $frame $id"

    $w configure -menu $mb

    # Dialog

    frame $w.basic -relief groove -borderwidth 2
    frame $w.ref -relief groove -borderwidth 2
    frame $w.dist -relief groove -borderwidth 2
    frame $w.buttons -relief groove -borderwidth 2
    pack $w.basic $w.ref $w.dist $w.buttons -fill x -ipadx 4 -ipady 4

    # ID

    label $w.basic.idTitle -text "Id"
    label $w.basic.idValue -text "$id"

    # Text

    label $w.basic.textTitle -text "Text"
    entry $w.basic.textValue -textvariable marker($frame,$id,text) \
	-width 40 -relief sunken -bd 2

    grid $w.basic.idTitle -row 0 -column 0 -padx 4 -sticky w
    grid $w.basic.idValue -row 0 -column 1 -padx 4 -sticky w

    grid $w.basic.textTitle -row 1 -column 0 -padx 4 -sticky w
    grid $w.basic.textValue -row 1 -column 1 -padx 4 -sticky w -columnspan 3

    # Points

    label $w.ref.title -text "Points"
    entry $w.ref.x -textvariable marker($frame,$id,x) \
	-width 13 -relief sunken -bd 2
    entry $w.ref.y -textvariable marker($frame,$id,y) \
	-width 13 -relief sunken -bd 2
    label $w.ref.coord -relief groove -bd 2 -width 9 -padx 4 \
	-textvariable marker($frame,$id,coord)

    entry $w.ref.x2 -textvariable marker($frame,$id,x2) \
	-width 13 -relief sunken -bd 2
    entry $w.ref.y2 -textvariable marker($frame,$id,y2) \
	-width 13 -relief sunken -bd 2

    grid $w.ref.title -row 1 -column 0 -padx 4 -sticky w
    grid $w.ref.x -row 1 -column 1 -padx 4 -sticky w
    grid $w.ref.y -row 1 -column 2 -padx 4 -sticky w
    grid $w.ref.coord -row 1 -column 3 -padx 4 -sticky w

    grid $w.ref.x2 -row 2 -column 1 -padx 4 -sticky w
    grid $w.ref.y2 -row 2 -column 2 -padx 4 -sticky w

    # Distance

    label $w.dist.dtitle -text "Distance"
    label $w.dist.dist -relief groove -bd 2 -width 13  \
	-textvariable marker($frame,$id,dist)
    label $w.dist.format -relief groove -bd 2 -width 8 -padx 4 \
	-textvariable marker($frame,$id,distformat)
    label $w.dist.atitle -text "Axis Dist"
    label $w.dist.x -relief groove -bd 2 -width 13  \
	-textvariable marker($frame,$id,distx)
    label $w.dist.y -relief groove -bd 2 -width 13  \
	-textvariable marker($frame,$id,disty)

    grid $w.dist.dtitle -row 3 -column 0 -padx 4 -sticky w
    grid $w.dist.dist -row 3 -column 1 -padx 4 -pady 2 -sticky w
    grid $w.dist.format -row 3 -column 3 -padx 4 -sticky w
    grid $w.dist.atitle -row 4 -column 0 -padx 4 -sticky w
    grid $w.dist.x -row 4 -column 1 -padx 4 -pady 2 -sticky w
    grid $w.dist.y -row 4 -column 2 -padx 4 -pady 2 -sticky w

    # Angle

    label $w.dist.angtitle -text "Angle"
    label $w.dist.angle -relief groove -bd 2 -width 13  \
	-textvariable marker($frame,$id,angle)
    label $w.dist.angformat -text "degrees" -relief groove \
	-bd 2 -width 8 -padx 4

    grid $w.dist.angtitle -row 5 -column 0 -padx 4 -sticky w
    grid $w.dist.angle -row 5 -column 1 -padx 4 -pady 2 -sticky w
    grid $w.dist.angformat -row 5 -column 3 -padx 4 -sticky w

    # Buttons

    button $w.buttons.apply -text "Apply" \
	-command "ApplyRulerDialog $frame $id"
    button $w.buttons.close -text "Close" \
	-command "CloseRulerDialog $frame $id"
    pack $w.buttons.apply $w.buttons.close -side left -padx 10 -expand true

    bind $w <Return> "ApplyRulerDialog $frame $id"
}

proc PropMenuRulerDialog {frame mb id} {
    global menu
    global marker

    menu $mb.properties -tearoff 0 -selectcolor $menu(selectcolor)
    $mb.properties add checkbutton -label "Can Edit" \
	-variable marker($frame,$id,edit) \
	-command "PropertyMarkerDialog $frame $id edit"
    $mb.properties add checkbutton -label "Can Move" \
	-variable marker($frame,$id,move) \
	-command "PropertyMarkerDialog $frame $id move"
    $mb.properties add checkbutton -label "Can Delete" \
	-variable marker($frame,$id,delete) \
	-command "PropertyMarkerDialog $frame $id delete"
}

proc ApplyRulerDialog {frame id} {
    global marker

    EditRulerDialog $frame $id
    $frame marker $id text [QuoteString "$marker($frame,$id,text)"]

    UpdateMarkerMenu
}

proc CloseRulerDialog {frame id} {
    global marker

    unset marker($frame,$id,x2)
    unset marker($frame,$id,y2)

    $frame marker $id delete callback edit EditRulerCB
    $frame marker $id delete callback move EditRulerCB
    $frame marker $id delete callback text TextMarkerCB
    $frame marker $id delete callback color ColorMarkerCB
    $frame marker $id delete callback property PropertyMarkerCB
    $frame marker $id delete callback font FontMarkerCB
    $frame marker $id delete callback delete DeleteRulerDialog

    DeleteMarkerDialog $frame $id
}

proc DeleteRulerDialog {frame id} {
    global marker

    unset marker($frame,$id,x2)
    unset marker($frame,$id,y2)

    set mb ".mb$id"
    destroy $mb.distformat

    DeleteMarkerDialog $frame $id
}

proc EditRulerDialog {frame id} {
    global marker

    $frame marker $id ruler point \
	$marker($frame,$id,x) $marker($frame,$id,y) \
	$marker($frame,$id,x2) $marker($frame,$id,y2) $marker($frame,$id,coord)

    set d [$frame get marker $id ruler distance]
    set marker($frame,$id,dist) [lindex $d 0]
    set marker($frame,$id,distx) [lindex $d 1]
    set marker($frame,$id,disty) [lindex $d 2]
    set marker($frame,$id,angle) [lindex $d 3]
}

proc EditRulerCB {frame id} {
    global marker

    set c $marker($frame,$id,coord)
    if {$c == "fk4" | $c == "fk5" | $c == "icrs" \
	    | $c == "galactic" | $c == "ecliptic"} {
	set p [$frame get marker $id ruler point \
			$marker($frame,$id,coord) $marker($frame,$id,format)]

	set marker($frame,$id,x) [lindex $p 0]
	set marker($frame,$id,y) [lindex $p 1]
	set marker($frame,$id,x2) [lindex $p 3]
	set marker($frame,$id,y2) [lindex $p 4]
    } else {
	set p [$frame get marker $id ruler point $marker($frame,$id,coord)]

	set marker($frame,$id,x) [lindex $p 0]
	set marker($frame,$id,y) [lindex $p 1]
	set marker($frame,$id,x2) [lindex $p 2]
	set marker($frame,$id,y2) [lindex $p 3]
    }

    set d [$frame get marker $id ruler distance]
    set marker($frame,$id,dist) [lindex $d 0]
    set marker($frame,$id,distx) [lindex $d 1]
    set marker($frame,$id,disty) [lindex $d 2]
    set marker($frame,$id,angle) [lindex $d 3]
}

proc CoordRulerCB {frame id} {
    global marker

    $frame marker $id ruler system \
	$marker($frame,$id,coord) $marker($frame,$id,distformat)
    EditRulerCB $frame $id
}
