#include "contour.h"
#include "frame.h"
#include "util.h"
#include "nan.h"

ExpPhyContourPlot::ExpPhyContourPlot(Frame* p, FitsImage* fits) : ContourPlot(p)
{
  // This routine produces contour plots.  It is a modified version of
  // the program in Appendis H of "Methods of Experimental Physics" Vol 12
  // Part C - Radio Observations    ed. by M.L.Meeks.
  // The original version was written by Norm Brenner and Stan Zisk
  // By Doug Mink, Harvard-Smithsonian Center for Astrophysics

  /*
  numLevel = 4;

  double level[numLevel];
  contours = new Vector**[numLevel];
  npts = new int[numLevel];

  int width = fits->width();
  int height = fits->height();

  // Find Min/Max of data

  double min = fits->getMinDouble();
  double max = fits->getMaxDouble();

  double step = (max-min)/(numLevel-1);
  for (int l=0; l<numLevel; l++)
    level[l] = l*step + min;

  // Compare to LevelMin/LevelMax

  if (min > level[numLevel-1] || max < level[0])
    return;

  // Loop through contour level

  for (int l=0; l<numLevel; l++) {
    npts[l] = 0;
    contours[l] = new Vector*[maxPoint];

    if (level[l]<min || level[l]>max)
      break;

    // Run through array 4 elements at a time

    double v[5];
    for (int j=1; j<height-1; j++) {
      if (fmod(j,100) == 0)
	cerr << j << ' ';
      int ij = width * (j - 1);
      for (int i=1; i<width-1; i++) {
	ij = ij + 1;
	int ij1 = ij + width;

	v[0] = fits->getValueDouble(ij1-1);
	v[1] = fits->getValueDouble(ij-1);
	v[2] = fits->getValueDouble(ij);
	v[3] = fits->getValueDouble(ij1);
	v[4] = fits->getValueDouble(ij1-1);

	// Check each edge of square of elements for crossings by current contour

	if (!isnand(v[0]) && !isnand(v[1]) && !isnand(v[2]) && !isnand(v[3])) {

	  for (int k=0; k<4; k++) {

	    // Look for contour crossing on this edge of square

	    if ((v[k] > level[l] && v[k+1] > level[l]) ||
		(v[k] < level[l] && v[k+1] < level[l]) ||
		(v[k] == v[k+1]))
	      break;
	    
	    // Compute location of crossing

	    double delta = (level[l] - v[k]) / (v[k+1] - v[k]);
	    double x,y;
	    switch (k) {
	    case 0:
	      x = i;
	      y = j - delta;
	      break;
	    case 1:
	      x = i + delta;
	      y = j;
	      break;
	    case 2:
	      x = i + 1;
	      y = j + delta;
	      break;
	    case 3:
	      x = i - delta;
	      y = j + 1;
	      break;
	    }
	    
	    // If contour passes through this set of four points, plot it
	    contours[l][npts[l]] = new Vector(x,y);
	    npts[l]++;
	  }
	}
      }
    }
    cerr << endl << "npts: " << npts[l] << endl;
  }
  */
}

/*
c*** File plotlib/plcon.f
c*** February 22, 1993
c*** By Doug Mink, Harvard-Smithsonian Center for Astrophysics

	Subroutine PLCON (IDATA,CMAX,NC,NR,IC0,IR0,LMIN,LMAX,LSTP,ILAB)

c--- This routine produces contour plots.  It is a modified version of
c    the program in Appendis H of "Methods of Experimental Physics" Vol 12
c    Part C - Radio Observations    ed. by M.L.Meeks.
c    The original version was written by Norm Brenner and Stan Zisk

	Integer*2 IDATA(*)
c			Address of first word of data to contour
	Integer*4 CMAX
c			Horizontal (=first) dimension of array IDATA
	Integer*4 NC,NR
c			Dimensions of segment of array to use
	Integer*4 IC0,IR0
c			Coordinates of IDATA in the big picture
	Integer*4 LMIN
c			First contour value
	Integer*4 LMAX
c			Last contour value
	Integer*4 LSTP
c			Contour interval
	Integer*4 ILAB
c			If =0, no labels; if >0, font size for labels

	Logical LABEL
	Character TEMP*20
	Integer*4 IVAL(5),IV0,IV1,IPL,NPT,LEV,NC1,NCHAR,NR1
	Real*4 X,Y,DVAL(5),DV0,DV1,CTR,DELTA,XZ,YZ
	Integer*4 I,ICORN,IDMAX,IDMIN,IER,IJ,IJ1,ILEV,IX,IXP,IY,IYP,J,K

	Include 'plx2.cmb'

	Common/PCON/ IXP,IYP

	IXP = 0
	IYP = 0

	XZ = Float (IC0)
	YZ = Float (IR0)
	Call PLOTX (0,XZ,YZ,IX,IY)
	Call PLALPH

	IDMIN = 32000
	IDMAX = 0
	Do 10 J = 1, NR
	   IJ = CMAX * (J - 1)
	   Do 10 I = 1, NC
	      IJ = IJ + 1
	      If (IDATA(IJ) .eq. 0) Go to 10
	      If (IDATA(IJ) .gt. IDMAX) IDMAX = IDATA(IJ)
	      If (IDATA(IJ) .lt. IDMIN) IDMIN = IDATA(IJ)
10            Continue
	If (IDMAX .lt. IDMIN) Return

	If ((IDMIN .gt. LMAX) .or. (IDMAX .lt. LMIN)) Return

	NC1 = NC - 1
	NR1 = NR - 1
	ILEV = 0

c  Loop through contour levels

	Do 140 LEV = LMIN, LMAX, LSTP
	    Call PLALPH
	    ILEV = ILEV + 1
	    If ((LEV .lt. IDMIN) .or. (LEV .gt. IDMAX)) Go to 140
	    If (Mod (ILEV,5) .eq. 1) Then
		Call PLLSIZ (2)
		LABEL = .true.
	    Elseif (Mod (ILEV,10) .eq. 1) Then
		Call PLLSIZ (3)
		LABEL = .true.
	    Else
		Call PLLSIZ (1)
		LABEL = .false.
	    Endif
	    If (ILAB .le. 0) LABEL = .false.
	    CTR = Float (LEV)

c  Run through array 4 elements at a time

	    Do 130 J = 1,NR1
	    IJ = CMAX * (J - 1)
	    Do 130 I = 1,NC1
		IJ = IJ + 1
		IJ1 = IJ + CMAX
		IVAL(1) = IDATA(IJ1)
		IVAL(2) = IDATA(IJ)
		IVAL(3) = IDATA(IJ+1)
		IVAL(4) = IDATA(IJ1+1)
		IVAL(5) = IDATA(IJ1)
		Do 40 K = 1, 5
		    If (IVAL(K) .eq. IVNULL) Go to 130
40		    DVAL(K) = IVAL(K)

c  Check each edge of square of elements for crossings by current contour

		NPT = 0
		IPL = 0
		Do 120 ICORN = 1,4
		    IV0 = IVAL(ICORN)
		    IV1 = IVAL(ICORN+1)

c		Look for contour crossing on this edge of square

		    If ((IV0 .gt. LEV) .and. (IV1 .gt. LEV)) Go to 120
		    If ((IV0 .lt. LEV) .and. (IV1 .lt. LEV)) Go to 120
		    If (IV0 .eq. IV1) Go to 120

c		Compute location of crossing

		    NPT = NPT + 1
		    DV0 = DVAL(ICORN)
		    DV1 = DVAL(ICORN+1)
		    DELTA = (CTR - DV0) / (DV1 - DV0)
		    If (ICORN .eq. 1) Then
			X = Float (IC0 + I)
			Y = Float (IR0 + J) + 1. - DELTA
		    Elseif (ICORN .eq. 2) Then
			X = Float(IC0 + I) + DELTA
			Y = Float (IR0 + J)
		    Elseif (ICORN .eq. 3) Then
			X = Float (IC0 + I + 1)
			Y = Float (IR0 + J) + DELTA
		    Else
			X = Float(IC0 + I) + 1. - DELTA
			Y = Float (IR0 + J + 1)
		    Endif

c		If contour passes through this set of four points, plot it

		    Call PLOTX (IPL,X,Y,IX,IY)
		    IPL = 1

120		    Continue

c  Label contour

		If (LABEL) Then
		    LABEL  =  .false.
		    Call BINASC (NCHAR,TEMP,-14,LEV,IER)
		    TEMP(NCHAR+1:NCHAR+1) = '$'
		    Call PLCSIZ (ILAB)
		    Call PLSTR (IX+5,IY+5,TEMP)
		    Endif

130		Continue

		Call PLALPH
140	    Continue

	Call PLLSIZ (1)
	Return
	End

c Nov 29 1984	Simplify labelling by using BINASC
c		Allow contour to cross pixel twice

c Feb  5 1986	Change to integer scaling arguments
c Jun  4 1986	Fix gap bug

c Feb  1 1988	Use ILAB as character size for labels
c		Add thicker lines every 5 or 10

c Feb 22 1993	Declare undeclared variables

*/

