#ifndef __point_h__
#define __point_h__

#include "marker.h"

class Point : public Marker {
 protected:
  char pointtype[16];

protected:
  virtual void render(Drawable, const Matrix&, GC) =0;
  virtual void dash(GC);
  virtual void psDash();

public:
  Point(Frame* p, const Vector& ctr, const char* clr, const char* f,
	const char* t, unsigned short prop);

  void updateBBox();
  void draw();
  void drawXOR();
  void drawMagnifier(const Matrix&);
  virtual void ps(int) =0; // generates postscript
  virtual void list(ostream&, int, int);

  void listSAOtng(ostream& str, int, int);
  void listPros(ostream& str, int, int);
  void listSAOimage(ostream&);
};

class CirclePoint : public Point {
private:
  void render(Drawable, const Matrix&, GC);

public:
  CirclePoint(Frame* p, const Vector& ctr, const char* clr, const char* f,
	      const char* t, unsigned short prop);
  void ps(int);
};

class BoxPoint : public Point {
private:
  void render(Drawable, const Matrix&, GC);

public:
  BoxPoint(Frame* p, const Vector& ctr, const char* clr, const char* f,
	   const char* t, unsigned short prop);
  void ps(int);
};

class DiamondPoint : public Point {
private:
  void render(Drawable, const Matrix&, GC);

public:
  DiamondPoint(Frame* p, const Vector& ctr, const char* clr, const char* f,
	       const char* t, unsigned short prop);
  void ps(int);
};

class CrossPoint : public Point {
private:
  void render(Drawable, const Matrix&, GC);

public:
  CrossPoint(Frame* p, const Vector& ctr, const char* clr,  const char* f,
	     const char* t, unsigned short prop);
  void ps(int);
};

class ExPoint : public Point {
private:
  void render(Drawable, const Matrix&, GC);

public:
  ExPoint(Frame* p, const Vector& ctr, const char* clr,  const char* f,
	  const char* t, unsigned short prop);
  void ps(int);
};

class ArrowPoint : public Point {
private:
  void render(Drawable, const Matrix&, GC);

public:
  ArrowPoint(Frame* p, const Vector& ctr, const char* clr,  const char* f,
	     const char* t, unsigned short prop);
  void ps(int);
};

class BoxCirclePoint : public Point {
private:
  void render(Drawable, const Matrix&, GC);

public:
  BoxCirclePoint(Frame* p, const Vector& ctr, const char* clr,  const char* f,
		 const char* t, unsigned short prop);
  void ps(int);
};

#endif
